;****************************************************************************
;*																			*
;*	memtool.asm - PIC source code for Memory Tool 							*
;*						ROM imager/EEPROM writer							*
;*																			*
;*	PIC type:		16F872													*
;*	Clock Speed:	4MHz													*
;*	Date:			20/04/07												*
;*	Author:			Nathan Dumont	(hairymnstr@gmail.com)					*
;*																			*
;*					Copyright (C)	2007, Nathan Dumont						*
;*																			*
;*	This program is free software; you can redistribute it and/or modify	*
;*	it under the terms of the GNU General Public License as published by	*
;*	the Free Software Foundation; either version 3 of the License, or		*
;*	(at your option) any later version.										*
;*																			*
;*	This program is distributed in the hope that it will be useful,			*
;*	but WITHOUT ANY WARRANTY; without even the implied warranty of			*
;*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			*
;*	GNU General Public License for more details.							*
;*																			*
;*	You should have received a copy of the GNU General Public License		*
;*	along with this program.  If not, see <http://www.gnu.org/licenses/>.	*
;*																			*
;*	Revision History:														*
;*		20/04/07	v1.0.0	Originalcode written in PICAXE BASIC			*
;*		06/08/07	v2.0.0	Complete re-write in MPASM						*
;*																			*
;****************************************************************************

list p=16f872			;PIC type definition
#include <p16f872.inc>	;Processor specific definitions

__config _XT_OSC & _LVP_OFF & _PWRTE_ON & _WDT_OFF & _BODEN_ON

;
;	PORTB I/O pin definitions
;
PORTBDIR equ B'00000000'	;PORTB start up direction
PORTBDEF equ B'11111111'	;PORTB start up values
;
USBRD	equ	0	;(output) RD line on USBMOD2
ADDRL	equ 1	;(output) Address low byte clock line
ADDRM	equ 2	;(output) Address middle byte clock line
MEMCS	equ 3	;(output) Memory chip select line
MEMRD	equ 4	;(output) Memory RD line
MEMWR	equ 5	;(output) Memory WR line
USBWR	equ 6	;(output) WR line on USBMOD2
ADDRH	equ 7	;(output) Address high byte clock line

;
;	PORTC I/O pin definitions
;
PORTCDIR equ B'11111111'	;PORTC start up direction
PORTCDEF equ B'00000000'	;PORTC start up values
;
RXF		equ 0	;(output) Receive buffer not empty signal from USBMOD2
READ	equ 1	;(input) Bit 0 of data bus, 
				;			on direction command 1=read, 0=write
TXE		equ 2	;(output) TX buffer full from USBMOD2

;
;	Main program - starts at memory address 0 since there are no 
;			interrupts used in this code.
;
ORG 0x00
init
	bsf STATUS,RP0	;select BANK1
		movlw PORTBDIR
		movwf TRISB		;set PORT directions
		movlw PORTCDIR
		movwf TRISC
	bcf STATUS,RP0	;select BANK0

	movlw PORTBDEF	;set PORT start up values
	movwf PORTB
	movlw PORTCDEF
	movwf PORTC

wait_lb				;wait for low address byte
	btfsc PORTC,RXF	;test for data available
	goto wait_lb	;if not loop
	bcf PORTB,USBRD	;read USBMOD2
	bcf PORTB,ADDRL	;Write low byte
	nop				;Make sure latch and PIC have time to get themselves sorted
	bsf	PORTB,ADDRL
	bsf PORTB,USBRD

wait_mb				;wait for middle address byte
	btfsc PORTC,RXF	;test for data available
	goto wait_mb	;if not loop
	bcf PORTB,USBRD	;read USBMOD2
	bcf PORTB,ADDRM	;Write middle byte
	nop				;Make sure latch and PIC have time to get themselves sorted
	bsf	PORTB,ADDRM
	bsf PORTB,USBRD

wait_hb				;wait for low address byte
	btfsc PORTC,RXF	;test for data available
	goto wait_hb	;if not loop
	bcf PORTB,USBRD	;read USBMOD2
	bcf PORTB,ADDRH	;Write high byte
	nop				;Make sure latch and PIC have time to get themselves sorted
	bsf	PORTB,ADDRH
	bsf PORTB,USBRD

wait_dir			;wait for direction instruction
	btfsc PORTC,RXF	;test for data available
	goto wait_dir	;if not loop
	bcf PORTB,USBRD	;read USBMOD2
	btfss PORTC,READ	;test for a read cycle
	goto write			;if not write
	bsf PORTB,USBRD
	
;
;	Do a read cycle, i.e. send a byte from memory to PC
;
	bcf PORTB,MEMCS	;select the memory chip
	bcf	PORTB,MEMRD	;tell chip to drive bus
wait_send
	btfsc PORTC,TXE	;check that the transmit buffer is available
	goto wait_send	;if not try again
	bcf PORTB,USBWR	;start a write cycle
	nop
	bsf PORTB,USBWR	
	bsf PORTB,MEMRD
	bsf PORTB,MEMCS	;set all the select lines high again
	goto wait_lb	;all done for this cycle so let's go again!

;
;	We've been asked to write the next byte to our parallel memory chip
;
write
	bsf PORTB,USBRD
	
wait_read
	btfsc PORTC,RXF	;wait for byte to arrive
	goto wait_read
	bcf PORTB,USBRD	;put the data from the USBMOD2 on the bus
	bcf PORTB,MEMCS	;select the memory chip
	bcf PORTB,MEMWR	;clock the WR line on the memory chip
	nop
	bsf PORTB,MEMWR ;finish WR cycle
	bsf PORTB,MEMCS
	bsf	PORTB,USBRD
	goto wait_lb	;see what the next byte will be, it's so exciting!

END